var path = require('path');
var os = require('os');

var DatabaseService = require('../database-service');

module.exports = {
    mockUser: function(id, pwd) {
        return {
            _id: id,
            _password: pwd,
            _roles: [],
            _variables: {
                USER_ID: id
            },
            setId: function(id) {
                this._id = id;
                this._variables.USER_ID = id;
            },
            getId: function() {
                return this._id;
            },
            setName: function(name) {
                this._name = name;
                this._variables.USER_NAME = name;
            },
            getName: function() {
                return this._name;
            },
            getPassword: function() {
                return this._password;
            },
            hasRole: function(roleId) {
                if (!roleId) {
                    return this._roles.length > 0;
                }
                return this._roles.index(roleId) !== -1;
            },
            setRoles: function(roles) {
                if (!roles) {
                    return;
                }
                if (typeof roles === 'number') {
                    roles = [roles + ''];
                } else if (typeof roles === 'string') {
                    roles = roles.replace(/(,|;)$/, '')
                         .split(/,|;/g)
                         .map(function(r) {
                            return r.replace(/\s/g, '');
                         });
                }
                this._roles = roles;
                this._variables.ROLES = roles.join(',');
                this._variables.ROLE_DISPLAY_NAMES = this._variables.ROLES; // bodhi.getRolesDisplayName(roles);
            },
            getRoles: function() {
                return this._roles;
            },
            getRoleIds: function() {
                return this._roles;
            },
            // getRolesDisplayName() {
            //     return bodhi.getRolesDisplayName(this._roles);
            // }
            setData: function(data) {
                this.setVariables(data);
            },
            setVariables: function(vars) {
                if (!vars || typeof vars !== 'object' || vars instanceof Array) {
                    console.warn('Invalid variables setting: ' + JSON.stringify(vars));
                    return;
                }
                for (var key in vars) {
                    this.setVariable(key, vars[key]);
                }
            },
            getVariables: function() {
                return this._variables || {};
            },
            setVariable: function(key, value) {
                if (!key) return null;

                key = key.toUpperCase();
                // ignore password
                if (/PWD|PASSWORD/.test(key)) {
                    return;
                }
                this._variables[key] = value;
            },
            getVariable: function(key) {
                if (!key) return null;
                key = (key + '').toUpperCase();
                return this._variables[key];
            }
        }
    },
    mockEnhancer: function(databaseConfiguration, serverVars, req, res) {
        return {
            getDatabaseService: function() {
                return new DatabaseService(databaseConfiguration);
            },
            getDatabaseConfig: function() {
                return databaseConfiguration;
            },
            getVariable: function(name) {
                name = name.toUpperCase();
                return serverVars[name];
            },
            setSessionVariable: function(name, value) {
                if (!req.session._variables) {
                    req.session._variables = {};
                }
                req.session._variables[name.toUpperCase()] = value;
            },
            getSessionVariable: function(name) {
                if (!name) {
                    return
                }
                return (req.session._variables || {})[name.toUpperCase()];
            },
            getContextRequest: function() {
                return req;
            },
            getContextResponse: function() {
                return res;
            },
            setData: function(key, value) {
                if (!key || typeof key !== 'string') {
                    throw new Error('The key must be a non-empty string.');
                }
                if (!this.data) {
                    this.data = {};
                }
                req.session['_mock_' + key] = value;
            },
            getData: function(key, callback) {
                if (!key || typeof key !== 'string') {
                    throw new Error('The key must be a non-empty string.');
                }
                if (typeof callback !== 'function') {
                    throw new Error('Can not get data whithout callback function');
                }
                callback(null, req.session['_mock_' + key]);
            },
            getUserFileDir: function() {
                return path.join(os.homedir(), 'enhancer/data');
            },
            resolveUserFilePath: function(url) {
                if (!url || typeof url !== 'string') {
                    return null;
                }
                var p = url.split(/\/file\//)[1];
                if (!p) {
                    return null;
                }
                var baseDir = this.getUserFileDir();
                return baseDir + '/' + p;
            }
        };
    }
}